@@ -7,7 +7,7 @@ use Class::Workflow::State::Simple;
 use Class::Workflow::Transition::Simple;
 use Class::Workflow::Instance::Simple;
 
-our $VERSION = "0.11_01";
+our $VERSION = "0.01";
 
 use Carp qw/croak/;
 use Scalar::Util qw/refaddr/;
@@ -37,210 +37,140 @@ sub new_instance {
 	$self->instance_class->new( %attrs );
 }
 
-has state_class => (
-        isa => "Str",
-        is  => "rw",
-        default => "Class::Workflow::State::Simple",
+use tt fields => [qw/state transition/];
+[% FOREACH field IN fields %]
+
+has [% field %]_class => (
+	isa => "Str",
+	is  => "rw",
+	default => "Class::Workflow::[% field | ucfirst %]::Simple",
 );
 
-has transition_class => (
-        isa => "Str",
-        is  => "rw",
-        default => "Class::Workflow::Transition::Simple",
+has _[% field %]s => (
+	isa => "HashRef",
+	is  => "ro",
+	default => sub { return {} },
 );
 
-has _state_fields => (
-        isa => "HashRef",
-        is  => "ro",
-        traits => [ 'Hash' ],
-        default => sub { return {} },
-        handles => {
-                states => 'values',
-                state_names => 'keys',
-                get_state => 'get',
-                get_states => 'get',
-                add_state => 'set',
-                delete_state => 'delete',
-                has_state => 'exists',
-        },
-);
-
-has _transition_fields => (
-        isa => "HashRef",
-        is  => "ro",
-        traits => [ 'Hash' ],
-        default => sub { return {} },
-        handles => {
-                transitions => 'values',
-                transition_names => 'keys',
-                get_transition => 'get',
-                get_transitions => 'get',
-                add_transition => 'set',
-                delete_transition => 'delete',
-                has_transition => 'exists',
-        },
-);
-
-sub rename_state {
-        my ( $self, $name, $new_name ) = @_;
-        my $obj = $self->delete_state( $name );
-        $obj->name( $new_name ) if $obj->can("name");
-        $self->add_state( $new_name => $obj );
+sub [% field %]s {
+	my $self = shift;
+	values %{ $self->_[% field %]s };
 }
 
-sub rename_transition {
-        my ( $self, $name, $new_name ) = @_;
-        my $obj = $self->delete_transition( $name );
-        $obj->name( $new_name ) if $obj->can("name");
-        $self->add_transition( $new_name => $obj );
+sub [% field %]_names {
+	my $self = shift;
+	keys %{ $self->_[% field %]s };
 }
 
-sub create_state {
-        my ( $self, $name, @attrs ) = @_;
-        $self->add_state( $name => $self->construct_state( @attrs ) );
+sub [% field %] {
+	my ( $self, @params ) = @_;
+
+	if ( @params == 1 ) {
+		if ( ref($params[0]) eq "HASH" ) {
+			@params = %{ $params[0] };
+		} elsif ( ref($params[0]) eq "ARRAY" ) {
+			@params = @{ $params[0] };
+		}
+	}
+
+	if ( !blessed($params[0]) and !blessed($params[1]) and @params % 2 == 0 ) {
+		# $wf->state( name => "foo", transitions => [qw/bar gorch/] )
+		return $self->create_or_set_[% field %]( @params );
+	} elsif ( !ref($params[0]) and @params % 2 == 1 ) {
+		# my $state = $wf->state("new", %attrs); # create new by name, or just get_foo
+		return $self->create_or_set_[% field %]( name => @params )
+	} elsif ( @params == 1 and blessed($params[0]) and $params[0]->can("name") ) {
+		# $wf->state( $state ); # set by object (if $object->can("name") )
+		return $self->add_[% field %]( $params[0]->name => $params[0] );
+	} elsif ( @params == 2 and blessed($params[1]) and !ref($params[0]) ) {
+		# $wf->state( foo => $state ); # set by name
+		return $self->add_[% field %]( @params );
+	} else {
+		if ( @params == 1 and blessed($params[0]) ) {
+			croak "The [% field %] $params[0] must support the 'name' method.";
+		} else {
+			croak "'[% field %]' was called with invalid parameters. Please consult the documentation.";
+		}
+	}
 }
 
-sub create_transition {
-        my ( $self, $name, @attrs ) = @_;
-        $self->add_transition( $name => $self->construct_transition( @attrs ) );
+sub get_[% field %] {
+	my ( $self, $name ) = @_;
+	$self->_[% field %]s->{$name}
 }
 
-sub construct_state {
-        my ( $self, %attrs ) = @_;
-        my $class = delete($attrs{class}) || $self->state_class;
-        $class->new( %attrs );
+sub get_[% field %]s {
+	my ( $self, @names ) = @_;
+	@{ $self->_[% field %]s }{@names}
 }
 
-sub construct_transition {
-        my ( $self, %attrs ) = @_;
-        my $class = delete($attrs{class}) || $self->transition_class;
-        $class->new( %attrs );
+sub add_[% field %] {
+	my ( $self, $name, $obj ) = @_;
+	
+	if ( exists $self->_[% field %]s->{$name} ) {
+		croak "$name already exists, delete it first."
+			unless refaddr($obj) == refaddr($self->_[% field %]s->{$name});
+		return $obj;
+	} else {
+		return $self->_[% field %]s->{$name} = $obj;
+	}
 }
 
-sub autovivify_states {
-        my ( $self, $thing ) = @_;
-
-        no warnings 'uninitialized';
-        if ( ref $thing eq "ARRAY" ) {
-                return [ map { $self->state( $_ ) } @$thing ];
-        } else {
-                return $self->state( $thing );
-        }
+sub rename_[% field %] {
+	my ( $self, $name, $new_name ) = @_;
+	my $obj = $self->delete_[% field %]( $name );
+	$obj->name( $new_name ) if $obj->can("name");
+	$self->add_[% field %]( $new_name => $obj );
 }
 
-sub autovivify_transitions {
-        my ( $self, $thing ) = @_;
-
-        no warnings 'uninitialized';
-        if ( ref $thing eq "ARRAY" ) {
-                return [ map { $self->transition( $_ ) } @$thing ];
-        } else {
-                return $self->transition( $thing );
-        }
+sub delete_[% field %] {
+	my ( $self, $name ) = @_;
+	delete $self->_[% field %]s->{$name};
 }
 
-sub create_or_set_state {
-        my ( $self, %attrs ) = @_;
-
-        my $name = $attrs{name} || croak "Every state must have a name";
-
-        $self->expand_attrs( \%attrs );
-
-        if ( my $obj = $self->get_state( $name ) ) {
-                delete $attrs{name};
-                foreach my $attr ( keys %attrs ) {
-                        $obj->$attr( $attrs{$attr} );
-                }
-
-                return $obj;
-        } else {
-                return $self->create_state( $name, %attrs );
-        }
+sub create_[% field %] {
+	my ( $self, $name, @attrs ) = @_;
+	$self->add_[% field %]( $name => $self->construct_[% field %]( @attrs ) );
 }
 
-sub create_or_set_transition {
-        my ( $self, %attrs ) = @_;
-
-        my $name = $attrs{name} || croak "Every transition must have a name";
-
-        $self->expand_attrs( \%attrs );
-
-        if ( my $obj = $self->get_transition( $name ) ) {
-                delete $attrs{name};
-                foreach my $attr ( keys %attrs ) {
-                        $obj->$attr( $attrs{$attr} );
-                }
-
-                return $obj;
-        } else {
-                return $self->create_transition( $name, %attrs );
-        }
+sub construct_[% field %] {
+	my ( $self, @attrs ) = @_;
+	$self->[% field %]_class->new( @attrs );
 }
 
-sub state {
-        my ( $self, @params ) = @_;
+sub autovivify_[% field %]s {
+	my ( $self, $thing ) = @_;
 
-        if ( @params == 1 ) {
-                if ( ref($params[0]) eq "HASH" ) {
-                        @params = %{ $params[0] };
-                } elsif ( ref($params[0]) eq "ARRAY" ) {
-                        @params = @{ $params[0] };
-                }
-        }
-
-        if ( !blessed($params[0]) and !blessed($params[1]) and @params % 2 == 0 ) {
-                # $wf->state( name => "foo", transitions => [qw/bar gorch/] )
-                return $self->create_or_set_state( @params );
-        } elsif ( !ref($params[0]) and @params % 2 == 1 ) {
-                # my $state = $wf->state("new", %attrs); # create new by name, or just get_foo
-                return $self->create_or_set_state( name => @params )
-        } elsif ( @params == 1 and blessed($params[0]) and $params[0]->can("name") ) {
-                # $wf->state( $state ); # set by object (if $object->can("name") )
-                return $self->add_state( $params[0]->name => $params[0] );
-        } elsif ( @params == 2 and blessed($params[1]) and !ref($params[0]) ) {
-                # $wf->state( foo => $state ); # set by name
-                return $self->add_state( @params );
-        } else {
-                if ( @params == 1 and blessed($params[0]) ) {
-                        croak "The state $params[0] must support the 'name' method.";
-                } else {
-                        croak "'state' was called with invalid parameters. Please consult the documentation.";
-                }
-        }
+	no warnings 'uninitialized';
+	if ( ref $thing eq "ARRAY" ) {
+		return [ map { $self->[% field %]( $_ ) } @$thing ];
+	} else {
+		return $self->[% field %]( $thing );
+	}
 }
 
-sub transition {
-        my ( $self, @params ) = @_;
+sub create_or_set_[% field %] {
+	my ( $self, %attrs ) = @_;
 
-        if ( @params == 1 ) {
-                if ( ref($params[0]) eq "HASH" ) {
-                        @params = %{ $params[0] };
-                } elsif ( ref($params[0]) eq "ARRAY" ) {
-                        @params = @{ $params[0] };
-                }
-        }
+	my $name = $attrs{name} || croak "Every [% field %] must have a name";
 
-        if ( !blessed($params[0]) and !blessed($params[1]) and @params % 2 == 0 ) {
-                # $wf->state( name => "foo", transitions => [qw/bar gorch/] )
-                return $self->create_or_set_transition( @params );
-        } elsif ( !ref($params[0]) and @params % 2 == 1 ) {
-                # my $state = $wf->state("new", %attrs); # create new by name, or just get_foo
-                return $self->create_or_set_transition( name => @params )
-        } elsif ( @params == 1 and blessed($params[0]) and $params[0]->can("name") ) {
-                # $wf->state( $state ); # set by object (if $object->can("name") )
-                return $self->add_transition( $params[0]->name => $params[0] );
-        } elsif ( @params == 2 and blessed($params[1]) and !ref($params[0]) ) {
-                # $wf->state( foo => $state ); # set by name
-                return $self->add_transition( @params );
-        } else {
-                if ( @params == 1 and blessed($params[0]) ) {
-                        croak "The transition $params[0] must support the 'name' method.";
-                } else {
-                        croak "'transition' was called with invalid parameters. Please consult the documentation.";
-                }
-        }
+	$self->expand_attrs( \%attrs );
+
+	if ( my $obj = $self->get_[% field %]( $name ) ) {
+		delete $attrs{name};
+		foreach my $attr ( keys %attrs ) {
+			$obj->$attr( $attrs{$attr} );
+		}
+
+		return $obj;
+	} else {
+		return $self->create_[% field %]( $name, %attrs );
+	}
 }
 
+[% END %]
+no tt;
+
 sub expand_attrs {
 	my ($self, $attrs ) = @_;
 
@@ -363,28 +293,21 @@ Class::Workflow - Light weight workflow system.
 Workflow systems let you build a state machine, with transitions between
 states.
 
-=head1 EXAMPLES
-
-There are several examples in the F<examples> directory, worth looking over to
-help you understand and to learn some more advanced things.
-
-The most important example is probably how to store a workflow definition (the
-states and transitions) as well as the instances using L<DBIx::Class> in a
-database.
-
-=head2 Bug Tracker Example
+=head1 BUG TRACKER EXAMPLE
 
 One of the simplest examples of a workflow which you've probably used is a bug
 tracking application:
 
 =over 4
 
-=item The initial state is 'new'
+The initial state is 'new'
 
 =item new
 
 New bugs arrive here.
 
+=over 4
+
 =item reject
 
 This bug is not valid.
@@ -397,6 +320,8 @@ This bug needs to be worked on.
 
 Target state: C<open>.
 
+=back
+
 =item rejected
 
 This is the state where deleted bugs go, it has no transitions.
@@ -405,6 +330,8 @@ This is the state where deleted bugs go, it has no transitions.
 
 The bug is being worked on right now.
 
+=over 4
+
 =item reassign
 
 Pass the bug to someone else.
@@ -417,20 +344,28 @@ The bug looks fixed, and needs verifification.
 
 Target state: C<awaiting_approval>.
 
+=back
+
 =item unassigned
 
 The bug is waiting for a developer to take it.
 
+=over 4
+
 =item take
 
 Volunteer to handle the bug.
 
 Target state: C<open>.
 
+=back
+
 =item awaiting_approval
 
 The submitter needs to verify the bug.
 
+=over 4
+
 =item resolved
 
 The bug is resolved and can be closed.
@@ -446,7 +381,7 @@ Target state: C<open>
 =item closed
 
 This is, like rejected, an end state (it has no transitions).
-
+gg
 =back
 
 If you read through this very simple state machine you can see that it
@@ -634,13 +569,19 @@ Persistence in workflows involves saving the workflow instance as a
 relationship of the item whose state it represents, or even treating the
 instance as the actual item.
 
-In any case, right now there are no turnkey persistence layers available.
+In any case, right now there are no persistence layers available. In the close
+future advanced L<DBIx::Class> persistence of both workflow definitions and
+instances will be available.
 
-A fully working L<DBIx::Class> example can be found in the F<examples/dbic>
-directory, but setup is manual. Serialization based persistence (with e.g.
-L<Storable>) is trivial as well.
+For the mean while I suggest you simply serialize the instance object's fields.
+If you need history, serialize recursively with C<prev> (although you may not
+want to do this every time).
 
-See L<Class::Workflow::Cookbook> for more details.
+Additionally the instance has the fields C<state> and C<transition>, which
+contain references to the current state and the transition which was applied to
+arrive at the current state. These two fields should probably be serialized as
+symbolic references (for example, C<< $instance->state->name >>), unless you
+want a copy some of the workflow definitions store in the instance as well.
 
 =head1 ROLES AND CLASSES
 
@@ -856,7 +797,7 @@ Yuval Kogman <nothingmuch@woobling.org>
 
 =head1 COPYRIGHT & LICENSE
 
-	Copyright (c) 2006-2008 Infinity Interactive, Yuval Kogman. All rights
+	Copyright (c) 2006 Infinity Interactive, Yuval Kogman. All rights
 	reserved. This program is free software; you can redistribute
 	it and/or modify it under the same terms as Perl itself.
 
